function displayArrow2D(originPoint, arrowVector, color)
%DISPLAYARROW Display 2D arrow.
%   DISPLAYARROW(ORIGINPOINT, ARROWVECTOR) displays an arrow in the current 
%   2D figure, starting at ORIGINPOINT (1x2 vector) and ending at
%   (ORIGINPOINT + ARROWVECTOR) (1x2 vector). A RGB color (1x3 vector) can
%   be given.
%
%   Also works with several vectors at a time, the arguments must then be
%   of size Nx2.

%   Author: Damien Teney

if nargin < 3
  color = 0.3 * [1 1 1]; % Set default value
end

% Remove arrows with very small length
%{
arrowsToKeep = getNormOfRows(arrowVector) > 2.0;
originPoint = originPoint(arrowsToKeep, :);
arrowVector = arrowVector(arrowsToKeep, :);
%}

% Method 1
%{
handle = quiver(originPoint(:, 1), originPoint(:, 2), arrowVector(:, 1), arrowVector(:, 2), 0, ...
  'filled', ...
  'color', [1 0 0], ...
  'linewidth', 1.0, ...
  'MaxHeadSize', 500, ...
  'MarkerSize', 50);
%}

% Method 2
%%{
% Find the position in the image to pass to the function annotation()
targetPoint = originPoint + arrowVector;
for i = 1:size(originPoint, 1); % For each arrow to display
  originPoint2 = getFigurePositionFromPlotPosition(originPoint(i, :));
  targetPoint2 = getFigurePositionFromPlotPosition(targetPoint(i, :));

  % Skip arrows that fall outside of the figure
  if any(originPoint2 < 0.0) || any(originPoint2 > 1.0), continue; end
  if any(targetPoint2 < 0.0) || any(targetPoint2 > 1.0) , continue; end

  % Display the arrow
  % See:
  % http://www.mathworks.com/help/matlab/ref/annotationarrowproperties.html
  annotation('arrow', [originPoint2(1) targetPoint2(1)], [originPoint2(2) targetPoint2(2)], ...
    'HeadStyle', 'vback1', ...
    'HeadWidth', 3.0, ...
    'HeadLength', 2.0, ...
    'LineWidth', 1.0, ...
    'Color', color);
end
%}

%==========================================================================

function figurePosition = getFigurePositionFromPlotPosition(plotPosition)
%GETFIGUREPOSITIONFROMXY Position in Matlab figure (values in [0, 1]) from a point in the current plot axes.

% Rename input
x = plotPosition(1);
y = plotPosition(2);

axisPosition = get(gca, 'Position');
xMinMax = xlim;
yMinMax = ylim;
yMinMax = fliplr(yMinMax); % Needed if we are in an image an not a plot (Y axis reversed then)
xx = axisPosition(1) + ((x - xMinMax(1)) / (xMinMax(2)-xMinMax(1))) * axisPosition(3);
yy = axisPosition(2) + ((y - yMinMax(1)) / (yMinMax(2)-yMinMax(1))) * axisPosition(4);

% Rename output
figurePosition = [xx yy];
